const { app, BrowserWindow, ipcMain, Tray, nativeImage, Menu, Notification } = require("electron");
const { env } = process;
const path = require("node:path");
const fs = require("node:fs");

let tray = null;
let mainWindow;
let isFullScreen = false;

const createWindow = () => {
  mainWindow = new BrowserWindow({
    width: 600,
    height: 800,
    webPreferences: {
      preload: path.join(__dirname, "preload.js"),
      nodeIntegration: false,
      contextIsolation: true,
      sandbox: true,
      backgroundThrottling: false,
      devTools: env.REMOBSTER_ENV === "dev",
    },
  });

  mainWindow.loadFile(path.join(__dirname, "../ui/index.html"));

  mainWindow.on("enter-full-screen", () => {
    console.log("entering full screen");
    isFullScreen = true;
  });

  mainWindow.on("leave-full-screen", () => {
    console.log("leaving full screen");
    isFullScreen = false;
  });
};

// Listen for events from the frontend
ipcMain.on("store-username-license", (event, data) => {
  console.log("store username!!");
  storeUsernameAndLicenseInSettingsFile(data);
});

ipcMain.on("update-tray-icon", (event, dataURL) => {
  const trayImage = nativeImage.createFromDataURL(dataURL);
  trayImage.setTemplateImage(true);
  tray.setImage(trayImage);
});

function storeUsernameAndLicenseInSettingsFile(data) {
  const userDataFile = path.join(app.getPath("userData"), "settings.json");
  fs.writeFileSync(userDataFile, JSON.stringify({ username: data.username, license: data.license }));
}

function readUsernameAndLicenseInSettingsFile() {
  const userDataFile = path.join(app.getPath("userData"), "settings.json");
  try {
    const data = fs.readFileSync(userDataFile, "utf8");
    return JSON.parse(data);
  } catch (error) {
    return undefined;
  }
}

function useLocalhost() {
  console.log("hello localhost");
  mainWindow.webContents.send("set-host", { host: "localhost:8080", protocol: "http://", wsProtocol: "ws://" });
}

function useProdHost() {
  console.log("hello prod");
  mainWindow.webContents.send("set-host", { host: "app.remobster.io", protocol: "https://", wsProtocol: "wss://" });
}

ipcMain.on("open-fullscreen", () => {
  if (process.platform !== "darwin") {
    mainWindow.resizable = true;
  }
  mainWindow.show();
  mainWindow.setAlwaysOnTop(true, "screen-saver");
  mainWindow.setAlwaysOnTop(false);
  mainWindow.setFullScreen(true);
});

ipcMain.on("close-fullscreen", () => {
  mainWindow.setFullScreen(false);
  if (process.platform !== "darwin") {
    mainWindow.resizable = false;
  }
});

app.whenReady().then(() => {
  createWindow();

  //platform specific behaviour for macOS, open new window if there is none.
  app.on("activate", () => {
    if (BrowserWindow.getAllWindows().length === 0) createWindow();
  });

  tray = new Tray(nativeImage.createFromPath(path.join(__dirname, "../clockTemplate.png")));
  tray.on("click", () => {
    mainWindow.show(); // If minimized, restore the window
    mainWindow.restore(); // If minimized, restore the window
    mainWindow.focus(); // Focus the window
  });

  const data = readUsernameAndLicenseInSettingsFile();
  if (data?.username && data?.license) {
    console.log("oh a username and a license", data.username, data.license);
    mainWindow.webContents.send("get-username-license", data);
  } else {
    console.log("no username and license saved");
  }

  console.log("What is the environment", env.REMOBSTER_ENV);
  environment = env.REMOBSTER_ENV;

  if (environment === "dev") {
    useLocalhost();
    mainWindow.webContents.openDevTools();
  } else {
    mainWindow.resizable = false;
    const menu = Menu.getApplicationMenu();
    const items = menu?.items.filter((item) => item.role !== "viewmenu");
    Menu.setApplicationMenu(Menu.buildFromTemplate(items));
    useProdHost();
  }
});

//platform specific behaviour for Windows and Linux, close app if the last window is closed.
app.on("window-all-closed", () => {
  if (process.platform !== "darwin") {
    app.quit();
  }
});
